#!/usr/bin/ruby

# active window can be found like this: xprop -root | grep ACTIVE_WIND | head -1

debug = false

unless debug
	# fork and detach
	pid = fork
	# the parent has to die, the pid-varaible contains the process-id of the child
	exit if pid 
	File.umask 0000
	STDIN.reopen "/dev/null"
	STDOUT.reopen "/dev/null", "a" 
	STDERR.reopen STDOUT
end

# open a socket so others can connect to this daemon

require 'socket'

socket_path = "/tmp/switch_app_#{ENV['USERNAME']}.sock"

File.delete(socket_path) if File.exist?(socket_path)

sock = UNIXServer.open(socket_path)
current_pid = 0
current_process_name = nil
pid_id_map = {}
while true
	socket_handle = sock.accept
	process_name = socket_handle.recvfrom(124).first
	fh = nil
	next_pressed = false
	if process_name == "__NEXT"
		next_pressed = true
		process_name = current_process_name
	end
	if process_name == "__NEXT"
		pid = current_pid
	else
		# first, get the PID of the asked-for process: 
		fh = IO.popen "ps -C #{process_name} -o pid="
		pid = 0
		if fh
			pid = fh.readlines.first.to_i
		end
		fh.close
	end
	# if no pid is found, there's a possibility that this is an nx-session for example, so try to find the window-name.
	no_pid = false
	if pid == 0 
		no_pid = true
		puts "executing: wmctrl -l -p | egrep '^0x[0-9a-f]+\s+\-?[01] [0-9]+\s+.*#{process_name}.*'"
		fh = IO.popen "wmctrl -l -p | egrep '^0x[0-9a-f]+\s+\-?[01] [0-9]+\s+.*#{process_name}.*'"
		current_pid = 0
	else 
		# now, get all the window-ids that correspond to that pid
		fh = IO.popen "wmctrl -l -p | egrep '^0x[0-9a-f]+\s+\-?[01] #{pid} '"
	end
	lines = fh.readlines
	fh.close

	# if no pid could be found (nx-session), use the pid from wmctrl. 
	if no_pid and lines.size > 0 
		wmctrl_output = lines.first.split(/ /)
		puts wmctrl_output.inspect
	elsif no_pid and lines.size == 0 
		# try to launch that program in that case. this must not be successful :) 
		puts "trying to launch #{process_name}"
		system("#{process_name} &")
	end
	window_ids = []
	# push all the window-ids onto the stack. 
	lines.each do |line|
		window_ids.push line.split(/ /).first
	end
	if not pid_id_map[process_name]
		pid_id_map[process_name] = { :current_window_id => window_ids.first, :window_ids => window_ids }
	else
		pid_id_map[process_name][:window_ids] = window_ids
		# if the previous window has been closed, we need to find the current window.
		if not window_ids.member? pid_id_map[process_name][:current_window_id]
			pid_id_map[process_name][:current_window_id] = window_ids.first
		end
	end
	if process_name == "__NEXT" or next_pressed
		current_window_id = pid_id_map[process_name][:current_window_id]
		window_ids = pid_id_map[process_name][:window_ids]
		index = window_ids.find_index current_window_id
		index = 0 unless index
		if index + 1 == window_ids.size
			index = -1
		end
		pid_id_map[process_name][:current_window_id] = window_ids[index + 1]
	end
	
	puts "switching to #{pid_id_map[process_name][:current_window_id]}"
	system "wmctrl -i -a #{pid_id_map[process_name][:current_window_id]}"
	current_pid = pid
	current_process_name = process_name
end

sock.close
